import jwt
from typing import List, Union, Dict, Any
from datetime import datetime

from django.utils.encoding import smart_text
from django.utils.translation import gettext_lazy as _

from rest_framework.authentication import get_authorization_header
from rest_framework.request import Request
from rest_framework.exceptions import AuthenticationFailed

from .settings import auth_settings


__all__ = [
    "AccessToken",
    "RefreshToken",
]


TokenType = Union["AccessToken", "RefreshToken"]


class AccessToken:

    token_type = "access"
    lifetime = auth_settings.ACCESS_TOKEN_LIFETIME

    def __init__(self, token: str = None, expected_claims: List[str] = None, type_check: bool = True):
        """Create a new token or construct one from encoded string.

        :param token: Encoded token without prefix.
        :param expected_claims: Verify that these custom claims are present.
        :param type_check: Check if token is of correct token type.
        :raises AuthenticationFailed: Token was invalid
        """
        if token:
            try:
                self.payload = jwt.decode(
                    jwt=token,
                    key=auth_settings.SIGNING_KEY,
                    options={
                        "require_exp": True,
                        "verify_exp": True,
                        "verify_aud": auth_settings.AUDIENCE is not None,
                        "verify_iss": auth_settings.ISSUER is not None,
                    },
                    leeway=auth_settings.LEEWAY,
                    audience=auth_settings.AUDIENCE,
                    issuer=auth_settings.ISSUER,
                    algorithms=[auth_settings.ALGORITHM],
                )
            except jwt.ExpiredSignatureError:
                raise AuthenticationFailed(_("Signature has expired."))
            except jwt.DecodeError:
                raise AuthenticationFailed(_("Error decoding signature."))
            except jwt.InvalidTokenError:
                raise AuthenticationFailed(_("Invalid token."))

            if expected_claims is not None:
                self.verify_payload(expected_claims)
            if type_check:
                self.verify_token_type()

        else:  # new token
            self.payload = {"type": self.token_type}
            self.renew()

            if auth_settings.AUDIENCE is not None:
                self.payload["aud"] = auth_settings.AUDIENCE
            if auth_settings.ISSUER is not None:
                self.payload["iss"] = auth_settings.ISSUER

    @classmethod
    def from_request(cls, request: Request, expected_claims: List[str] = None):
        """Construct a token from request Authorization header.

        :param request: Request with Authorization header.
        :param expected_claims: Verify that these custom claims are present.
        :raises AuthenticationFailed: Request header or token was invalid.
        """
        try:
            prefix, encoded_token = get_authorization_header(request).split()
        except ValueError:
            raise AuthenticationFailed(_("Invalid Authorization header."))

        if smart_text(prefix).lower() != auth_settings.HEADER_PREFIX.lower():
            raise AuthenticationFailed(_("Invalid prefix."))

        return cls(token=encoded_token, expected_claims=expected_claims)

    def __repr__(self) -> str:
        return repr(self.payload)

    def __str__(self) -> str:
        return jwt.encode(
            payload=self.payload,
            key=auth_settings.SIGNING_KEY,
            algorithm=auth_settings.ALGORITHM,
            headers=auth_settings.EXTRA_HEADERS,
        )

    def __getitem__(self, key: str) -> Any:
        return self.payload[key]

    def __setitem__(self, key: str, value: Union[int, float, str, bool, bytes]) -> None:
        self.payload[key] = value

    def __delitem__(self, key: str) -> None:
        del self.payload[key]

    def __contains__(self, key: str) -> bool:
        return key in self.payload

    def get(self, key: str, default: str = None) -> Any:
        return self.payload.get(key, default)

    def verify_payload(self, expected_claims: List[str]) -> None:
        for claim in expected_claims:
            if claim not in self:
                raise AuthenticationFailed(_("Missing claims."))

    def verify_token_type(self) -> None:
        if self.token_type != self.payload.get("type", "notype"):
            raise AuthenticationFailed(_("Invalid token type."))

    def sync_with(self, token: TokenType) -> None:
        """Sync this token's expiry and issuing times to the other token's.
        NOTE: THIS WILL CHANGE THE ENCODED TOKEN, SO BE SURE TO SAVE IT!
        """
        self.payload["exp"] = token["iat"] + self.lifetime
        self.payload["iat"] = token["iat"]

    def renew(self) -> None:
        """Renew token expiration.
        NOTE: THIS WILL CHANGE THE ENCODED TOKEN, SO BE SURE TO SAVE IT!
        """
        self.payload["exp"] = datetime.utcnow() + self.lifetime
        self.payload["iat"] = datetime.utcnow()


class RefreshToken(AccessToken):

    token_type = "refresh"
    lifetime = auth_settings.REFRESH_TOKEN_LIFETIME

    def new_access_token(self) -> "AccessToken":
        access = AccessToken()

        for claim, value in self.payload.items():
            if claim in ("aud", "iss", "exp", "iat", "type"):
                continue
            access[claim] = value

        return access

    def update(self, data: Dict[str, str] = None, **kwargs: str) -> None:
        """Update payload.
        NOTE: THIS WILL CHANGE THE ENCODED TOKEN, SO BE SURE TO SAVE IT!
        """
        self.payload.update({} if data is None else data, **kwargs)



from django.utils.translation import gettext_lazy as _

from rest_framework.exceptions import APIException
from rest_framework import status


__all__ = [
    "EmailServerException",
    "LoginCodeStillValid",
    "CorruptedDataException",
]


class EmailServerException(APIException):
    status_code = status.HTTP_503_SERVICE_UNAVAILABLE
    default_detail = _("Email server did not respond.")
    default_code = "email_server_down"


class LoginCodeStillValid(APIException):
    status_code = status.HTTP_409_CONFLICT
    default_detail = _("A login code for this email is still valid.")
    default_code = "login_still_valid"


class CorruptedDataException(APIException):
    status_code = status.HTTP_410_GONE
    default_detail = _("Data was corrupted.")
    default_code = "data_corruption"



import logging
from hashlib import md5
from inspect import cleandoc
from random import randint

from django.core.cache import cache
from django.core.mail import send_mail
from django.template.loader import render_to_string
from ipware import get_client_ip
from rest_framework.request import Request

from .settings import auth_settings


__all__ = [
    "generate_cache_key",
    "user_login_blocked",
    "send_login_email",
]


logger = logging.getLogger(__name__)


def random_code() -> str:
    return str(randint(1, 999_999)).zfill(6)


def generate_cache_key(content: str) -> str:
    """Generate cache key using a prefix (from auth_settings), and md5 hexdigest."""
    return f"{auth_settings.CACHE_PREFIX}-{md5(content.encode()).hexdigest()}"


def default_login_data() -> dict:
    """Default login data function. It is meant to be overriden in Django settings."""
    return {}


def validate(email: str) -> dict:
    """Default function to validate login. It is meant to be overriden in Django settings."""
    return {"email": email}


def user_login_blocked(request: Request) -> bool:

    ip, is_routable = get_client_ip(
        request=request,
        proxy_order=auth_settings.PROXY_ORDER,
        proxy_count=auth_settings.PROXY_COUNT,
        proxy_trusted_ips=auth_settings.PROXY_TRUSTED_IPS,
        request_header_order=auth_settings.REQUEST_HEADER_ORDER,
    )

    cache_key = generate_cache_key(ip)
    attempts = cache.get(cache_key, 0) + 1
    cache.set(cache_key, attempts, auth_settings.LOGIN_COOLDOWN.total_seconds())

    block = attempts >= auth_settings.LOGIN_ATTEMPTS
    wasnt_blocked = attempts - 1 < auth_settings.LOGIN_ATTEMPTS

    if block and wasnt_blocked:
        logger.warning(f"Blocked user with ip '{ip}' due to too many login attempts.")

    return block


def send_login_email(request: Request, code: str, email: str) -> None:

    plain_message = cleandoc(
        auth_settings.LOGIN_EMAIL_MESSAGE.format(
            code=code, valid=auth_settings.LOGIN_CODE_LIFETIME.total_seconds() // 60
        )
    )

    html_message = None
    if auth_settings.LOGIN_EMAIL_HTML_TEMPLATE is not None:
        html_message = render_to_string(
            auth_settings.LOGIN_EMAIL_HTML_TEMPLATE,
            context={"code": code, "valid": auth_settings.LOGIN_CODE_LIFETIME.total_seconds() // 60},
            request=request,
        )

    if auth_settings.SEND_EMAILS:
        send_mail(
            subject=auth_settings.LOGIN_SUBJECT_LINE,
            message=plain_message,
            from_email=auth_settings.LOGIN_SENDING_EMAIL,
            recipient_list=[email],
            html_message=html_message,
        )
    else:
        logger.info(plain_message)



import logging

from rest_framework.permissions import BasePermission
from rest_framework.exceptions import AuthenticationFailed

from .tokens import AccessToken
from .settings import auth_settings


__all__ = [
    "HasValidJWT",
]


logger = logging.getLogger(__name__)


class HasValidJWT(BasePermission):
    def has_permission(self, request, view):
        try:
            AccessToken.from_request(request, expected_claims=auth_settings.EXPECTED_CLAIMS)
        except AuthenticationFailed as f:
            logger.debug(f)
            return False

        return True



from typing import TYPE_CHECKING
from uuid import uuid4

from django.contrib.auth.models import AnonymousUser
from django.utils.functional import cached_property


if TYPE_CHECKING:
    from .tokens import AccessToken


__all__ = ["StatelessUser"]


class StatelessUser(AnonymousUser):
    """
    User that is not actually logged in, but enables
    authentication and permission checks.
    """

    is_active = True
    username = "StatelessUser"

    def __init__(self, token: "AccessToken" = None):
        self.token = token if token is not None else {}

    @cached_property
    def id(self):
        return uuid4()

    @cached_property
    def pk(self):
        return self.id

    @property
    def is_authenticated(self):
        return True

    def __str__(self):
        return self.__class__.__name__

    def __eq__(self, other):
        return self.id == other.id

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(str(self.token))



"""Default settings for the JWT email authentication."""

import os
from datetime import timedelta
from warnings import warn

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey
from cryptography.hazmat.primitives.serialization import load_ssh_private_key
from django.conf import settings
from django.test.signals import setting_changed
from rest_framework.settings import APISettings


__all__ = [
    "auth_settings",
]


# DO NOT USE IN PRODUCTION!
_DEFAULT_PUBLIC_KEY = "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIIMOFDpS02jVpNbJidXBM+s9QzWqVx56pxZdWEgVjA4T"
_DEFAULT_PRIVATE_KEY = (
    "-----BEGIN OPENSSH PRIVATE KEY-----|"
    "b3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAAAMwAAAAtzc2gtZW|"
    "QyNTUxOQAAACCDDhQ6UtNo1aTWyYnVwTPrPUM1qlceeqcWXVhIFYwOEwAAAJDEf7enxH+3|"
    "pwAAAAtzc2gtZWQyNTUxOQAAACCDDhQ6UtNo1aTWyYnVwTPrPUM1qlceeqcWXVhIFYwOEw|"
    "AAAECjUueNb+pa9Mf0cVahpJzyBbwQgZrp2qLgYykEiC4g4IMOFDpS02jVpNbJidXBM+s9|"
    "QzWqVx56pxZdWEgVjA4TAAAAC2xhbXBwQEtBTlRPAQI=|"
    "-----END OPENSSH PRIVATE KEY-----"
)


def load_signing_key() -> "Ed25519PrivateKey":
    key = os.environ.get("SIGNING_KEY", _DEFAULT_PRIVATE_KEY)
    if key == _DEFAULT_PRIVATE_KEY:
        warn(
            "Using the default signing key. "
            "Please change before going to production. "
            "To change, set 'SIGNING_KEY' environment variable."
        )
    key = "\n".join(key.split("|"))
    return load_ssh_private_key(key.encode(), password=None, backend=default_backend())


USER_SETTINGS = getattr(settings, "JWT_EMAIL_AUTH", None)

DEFAULTS = {
    "SEND_EMAILS": False,  # bool: Send email, off by default
    "SKIP_CODE_CHECKS": False,  # bool: When True, any code will work in login
    "SIGNING_KEY": load_signing_key(),  # str: JWT signing key
    "ACCESS_TOKEN_LIFETIME": timedelta(minutes=5),  # timedelta: How long a access token is valid for
    "REFRESH_TOKEN_LIFETIME": timedelta(days=14),  # timedelta: How long a refresh token is valid for
    "LOGIN_CODE_LIFETIME": timedelta(minutes=5),  # timedelta: How long a login code is stored in cache
    "VALIDATION_CALLBACK": "jwt_email_auth.utils.validate",  # callable -> dict: Function to use for validating use from email
    "LOGIN_DATA": "jwt_email_auth.utils.default_login_data",  # callable -> dict: Function to run to gather login data
    "CODE_GENERATOR": "jwt_email_auth.utils.random_code",  # callable -> str: Function to generate a login code
    "LOGIN_SENDING_EMAIL": None,  # str: Email sender. Default is settings.DEFAULT_FROM_EMAIL
    "LOGIN_SUBJECT_LINE": "Login to Django",  # str: Email subject line
    "LOGIN_EMAIL_MESSAGE": (  # str: Message to send in email. Must have {code} and {valid}!
        """
            Your login code:

            {code}

            This code is valid for the next {valid} minutes.
        """
    ),
    "LOGIN_EMAIL_HTML_TEMPLATE": None,  # str: Path to html_message template. Context must have {{ code }} and {{ valid }}!
    #
    # Encoding and decoding options:
    "ISSUER": None,  # str: Issuer of the JWT
    "AUDIENCE": None,  # str: Intended recipient of the JWT
    "LEEWAY": 0,  # int: A time margin in seconds for the expiration check
    "ALGORITHM": "EdDSA",  # str: Algorithm to sign and decrypt the token with
    "HEADER_PREFIX": "Bearer",  # str: Authorization: <HEADER_PREFIX> <token>
    "EXTRA_HEADERS": None,  # dict: Additional JWT header fields
    "EXPECTED_CLAIMS": [],  # list[str]: List of expected JWT content
    #
    # IP address spoofing prevention settings:
    # https://github.com/un33k/django-ipware/blob/master/README.md#advanced-users
    "PROXY_ORDER": "left-most",  # str: "left-most" or "right-most"
    "PROXY_COUNT": None,  # int: Number of proxies between the server and internet
    "PROXY_TRUSTED_IPS": None,  # list[str]: Only these proxy IPs are allowed connections
    "REQUEST_HEADER_ORDER": None,  # list[str]: Meta precedence order
    "CACHE_PREFIX": "Django",  # str: Cache prefix for login codes and banned IPs
    "LOGIN_ATTEMPTS": 10,  # int: Number of login attempts until banned
    "LOGIN_COOLDOWN": timedelta(minutes=5),  # timedelta: How long until login ban lifted
}

# List of settings that may be in string dot import notation.
IMPORT_STRINGS = [
    "VALIDATION_CALLBACK",
    "LOGIN_DATA",
    "CODE_GENERATOR",
]

auth_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)


def reload_api_settings(*args, **kwargs):
    global auth_settings

    setting, value = kwargs["setting"], kwargs["value"]

    if setting == "JWT_EMAIL_AUTH":
        auth_settings = APISettings(value, DEFAULTS, IMPORT_STRINGS)


setting_changed.connect(reload_api_settings)



from rest_framework.fields import CharField

from .tokens import AccessToken, RefreshToken
from .settings import auth_settings


__all__ = [
    "AutoTokenField",
]


class AutoTokenField(CharField):
    """Field where the JWT can be added from authorization header"""

    def __init__(self, **kwargs):
        key = kwargs.pop("return_key", None)
        self.refresh_token = kwargs.pop("refresh_token", False)
        super().__init__(**kwargs)
        self.return_key = key or self.field_name

    def to_internal_value(self, data):
        """Try to construct an Access token, or Refresh token if specified.

        :raises AuthenticationFailed: JWT token expired or malformed.
        """
        if self.refresh_token:
            return RefreshToken(token=data, expected_claims=auth_settings.EXPECTED_CLAIMS)
        return AccessToken(token=data, expected_claims=auth_settings.EXPECTED_CLAIMS)

    def to_representation(self, value):
        """Construct the JWT for outgoing data."""

        # Change the key under which the token is returned under
        self.field_name = self.return_key

        # Special case when refresh token is given,
        # should return new access token instead
        if self.refresh_token:
            return str(value.new_access_token())
        return str(value)



import logging

from django.core.cache import cache
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers, status
from rest_framework.exceptions import AuthenticationFailed, NotFound, PermissionDenied
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.views import APIView

from .exceptions import CorruptedDataException, EmailServerException, LoginCodeStillValid
from .serializers import LoginSerializer, RefreshTokenSerializer, SendLoginCodeSerializer
from .settings import auth_settings
from .tokens import RefreshToken
from .utils import generate_cache_key, send_login_email, user_login_blocked


__all__ = [
    "SendLoginCodeView",
    "LoginView",
    "RefreshTokenView",
]


logger = logging.getLogger(__name__)


class SendLoginCodeView(APIView):
    """Send a new login code to the email POST-ed here.

    HTTP 400 Bad Request: Email not given or type somehow invalid.

    HTTP 409 Conflict: There is already a login code valid for this email.

    HTTP 503 Service Unavailable: Email server could not send email.
    """

    authentication_classes = []
    permission_classes = []
    serializer_class: serializers.Serializer = SendLoginCodeSerializer

    def post(self, request: Request, *args, **kwargs) -> Response:
        login_info = self.serializer_class(data=request.data)
        login_info.is_valid(raise_exception=True)
        data = login_info.data

        cache_key = generate_cache_key(data["email"])
        if cache.get(cache_key, None) is not None:
            return Response(status=status.HTTP_204_NO_CONTENT)

        login_data = auth_settings.LOGIN_DATA()
        login_data["code"] = auth_settings.CODE_GENERATOR()
        logger.info(login_data)
        cache.set(cache_key, login_data, auth_settings.LOGIN_CODE_LIFETIME.total_seconds())

        try:
            send_login_email(self.request, code=login_data["code"], email=data["email"])
        except Exception as error:
            cache.delete(cache_key)
            logger.critical(error)
            raise EmailServerException(_("Failed to send login codes. Try again later."))

        return Response(status=status.HTTP_204_NO_CONTENT)


class LoginView(APIView):
    """Get new refresh and access token pair from a login code and email.

    HTTP 400 Bad Request: Email or code not given or their types are somehow invalid.

    HTTP 403 Forbidden: User has been blocked aftr too many attemps at login.

    HTTP 404 Not Found: No login code found for given email.

    HTTP 410 Gone: Login data was corrupted.
    """

    authentication_classes = []
    permission_classes = []
    serializer_class: serializers.Serializer = LoginSerializer

    def post(self, request: Request, *args, **kwargs) -> Response:
        if user_login_blocked(request):
            raise PermissionDenied(
                _("Maximum number of attempts reached. Try again in %(x)s minutes.")
                % {"x": auth_settings.LOGIN_COOLDOWN.total_seconds() // 60}
            )

        login = self.serializer_class(data=request.data)
        login.is_valid(raise_exception=True)
        data = login.data

        cache_key = generate_cache_key(data["email"])
        login_info = cache.get(cache_key, None)
        if login_info is None:
            raise NotFound(_("No login code found code for '%(email)s'.") % {"email": data["email"]})

        if not auth_settings.SKIP_CODE_CHECKS:
            if login_info.pop("code", None) != data["code"]:
                raise AuthenticationFailed(_("Incorrect login code."))

        refresh = RefreshToken()
        try:
            refresh.update({key: login_info[key] for key in auth_settings.EXPECTED_CLAIMS})
        except KeyError:
            logger.warning(
                "Some data was missing from saved login info. If you set EXPECTED_CLAIMS, you "
                "should provide a custom LOGIN_DATA function that returns them in a dict."
            )
            cache.delete(cache_key)
            raise CorruptedDataException(_("Data was corrupted. Try to send another login code."))

        cache.delete(cache_key)

        refresh = RefreshToken()
        access = refresh.new_access_token()
        access.sync_with(refresh)

        data = {"access": str(access), "refresh": str(refresh)}

        return Response(data=data, status=status.HTTP_202_ACCEPTED)


class RefreshTokenView(APIView):
    """Get new access token by POST-ing refresh token here.

    HTTP 400 Bad Request: Token not given or type somehow invalid.

    HTTP 401 Unauthorized: Refresh token has expired or is invalid.
    """

    authentication_classes = []
    permission_classes = []
    serializer_class: serializers.Serializer = RefreshTokenSerializer

    def post(self, request: Request, *args, **kwargs) -> Response:
        token = self.serializer_class(data=request.data)
        token.is_valid(raise_exception=True)
        data = token.data

        refresh = RefreshToken(data["token"], expected_claims=auth_settings.EXPECTED_CLAIMS)
        access = refresh.new_access_token()
        data = {"access": str(access)}

        return Response(data=data, status=status.HTTP_200_OK)



from rest_framework import serializers
from rest_framework.fields import empty

from .settings import auth_settings
from .tokens import AccessToken


__all__ = [
    "BaseAccessSerializer",
    "RefreshTokenSerializer",
    "LoginSerializer",
    "SendLoginCodeSerializer",
]


class SendLoginCodeSerializer(serializers.Serializer):
    email = serializers.EmailField(help_text="Email address to send the code to.")

    def validate(self, attrs):
        return auth_settings.VALIDATION_CALLBACK(email=attrs["email"])


class LoginSerializer(serializers.Serializer):
    code = serializers.CharField(help_text="Login code.")
    email = serializers.EmailField(help_text="Email address the code was sent to.")


class RefreshTokenSerializer(serializers.Serializer):
    token = serializers.CharField(help_text="Refresh token.")


class BaseAccessSerializer(serializers.Serializer):
    """Base serializer that adds hidden token field, and takes specified claims from it."""

    take_form_token: list[str] = []  # from authentication.settings.EXPECTED_CLAIMS
    """List of keys to take from token claims and pass to bound method."""

    def __init__(self, instance=None, data=empty, **kwargs):
        super().__init__(instance=instance, data=data, **kwargs)
        self.initial_data = self.add_token_claims(getattr(self, "initial_data", {}))

    def add_token_claims(self, data: dict) -> dict:
        token = AccessToken.from_request(
            request=self.context.get("request"),
            expected_claims=auth_settings.EXPECTED_CLAIMS,
        )
        data.update({key: token[key] for key in self.take_form_token})
        return data

    def to_internal_value(self, data: dict) -> dict:
        ret = super().to_internal_value(data)
        return self.add_token_claims(ret)

    def to_representation(self, instance) -> dict:
        ret = super().to_representation(instance)
        return self.add_token_claims(ret)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jwt_email_auth']

package_data = \
{'': ['*'], 'jwt_email_auth': ['locale/fi/LC_MESSAGES/*']}

install_requires = \
['Django>=3.2.9,<4.0.0',
 'PyJWT>=2.3.0,<3.0.0',
 'cryptography>=36.0.0,<37.0.0',
 'django-ipware>=4.0.0,<5.0.0',
 'djangorestframework>=3.12.2,<4.0.0']

setup_kwargs = {
    'name': 'jwt-email-auth',
    'version': '0.1.0',
    'description': 'JWT authentication from email login codes.',
    'long_description': "# JSON Web Token Email Authentiation\n\n[![Coverage Status](https://coveralls.io/repos/github/MrThearMan/jwt-email-auth/badge.svg?branch=main)](https://coveralls.io/github/MrThearMan/jwt-email-auth?branch=main)\n[![GitHub Workflow Status](https://img.shields.io/github/workflow/status/MrThearMan/jwt-email-auth/Tests)](https://github.com/MrThearMan/jwt-email-auth/actions/workflows/main.yml)\n[![PyPI](https://img.shields.io/pypi/v/jwt-email-auth)](https://pypi.org/project/jwt-email-auth)\n[![GitHub](https://img.shields.io/github/license/MrThearMan/jwt-email-auth)](https://github.com/MrThearMan/jwt-email-auth/blob/main/LICENSE)\n[![GitHub last commit](https://img.shields.io/github/last-commit/MrThearMan/jwt-email-auth)](https://github.com/MrThearMan/jwt-email-auth/commits/main)\n[![GitHub issues](https://img.shields.io/github/issues-raw/MrThearMan/jwt-email-auth)](https://github.com/MrThearMan/jwt-email-auth/issues)\n\n\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/jwt-email-auth)](https://pypi.org/project/jwt-email-auth)\n[![PyPI - Django Version](https://img.shields.io/pypi/djversions/jwt-email-auth)](https://pypi.org/project/jwt-email-auth)\n[![Custom - Django REST Framework Version](https://img.shields.io/badge/drf%20versions-3.7%20%7C%203.8%20%7C%203.9%20%7C%203.10%20%7C%203.11%20%7C%203.12-blue)](https://pypi.org/project/jwt-email-auth)\n\n```shell\npip install jwt-email-auth\n```\n---\n\n**Documentation**: [https://mrthearman.github.io/jwt-email-auth/](https://mrthearman.github.io/jwt-email-auth/)\n\n**Source Code**: [https://github.com/MrThearMan/jwt-email-auth](https://github.com/MrThearMan/jwt-email-auth)\n\n---\n\nThis module enables JSON Web Token Authentication in Django Rest framework without using Django's User model.\nInstead, login information is stored in [cache](https://docs.djangoproject.com/en/3.2/topics/cache/#the-low-level-cache-api),\na login code is sent to the user's email inbox, and then the cached information is obtained\nusing the code that was sent to the given email.\n",
    'author': 'Matti Lamppu',
    'author_email': 'lamppu.matti.akseli@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/MrThearMan/jwt-email-auth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)



import logging

from rest_framework.exceptions import AuthenticationFailed
from rest_framework.authentication import BaseAuthentication

from .settings import auth_settings
from .tokens import AccessToken
from .models import StatelessUser


__all__ = [
    "JWTAuthentication",
]


logger = logging.getLogger(__name__)


class JWTAuthentication(BaseAuthentication):
    def authenticate(self, request):
        try:
            token = AccessToken.from_request(request, expected_claims=auth_settings.EXPECTED_CLAIMS)
            return StatelessUser(token), None
        except AuthenticationFailed as f:
            logger.debug(f)
            return

    def authenticate_header(self, request):
        return f'{auth_settings.HEADER_PREFIX} realm="api"'
